from . import google_translator_toolkit
from . import google_sheets
from grow.common import utils
from grow.extensions import extension_importer


_kinds_to_classes = {}
_builtins = (
    google_translator_toolkit.GoogleTranslatorToolkitTranslator,
    google_sheets.GoogleSheetsTranslator,
)


def install_translator(translator):
    _kinds_to_classes[translator.KIND] = translator


def install_builtins():
    global _destination_kinds_to_classes
    for builtin in _builtins:
        install_translator(builtin)


def create_translator(pod, kind, config, project_title=None, instructions=None):
    install_builtins()
    if kind not in _kinds_to_classes:
        raise ValueError('No translator exists: "{}"'.format(kind))
    translator = _kinds_to_classes[kind]
    return translator(pod=pod, config=config,
                      project_title=project_title, instructions=instructions)


def register_extensions(extension_paths, pod_root):
    for path in extension_paths:
        cls = extension_importer.ExtensionImporter.find_extension(
            path, pod_root)
        install_translator(cls)



"""Subcommand for displaying pod stats."""

import os
import click
from grow.commands import shared
from grow.common import rc_config
from grow.deployments import stats as stats_lib
from grow.pods import pods
from grow import storage


CFG = rc_config.RC_CONFIG.prefixed('grow.inspect.stats')


@click.command(name='stats')
@shared.pod_path_argument
@click.option('--full/--no-full', '-f', is_flag=CFG.get('full', True), default=False,
              help='Whether to show full stats. By default, only '
                   'short stats are displayed. Short stats do not '
                   'require a build and may be faster to generate.')
def inspect_stats(pod_path, full):
    """Displays statistics about the pod."""
    root = os.path.abspath(os.path.join(os.getcwd(), pod_path))
    pod = pods.Pod(root, storage=storage.FileStorage)
    try:
        with pod.profile.timer('grow_inspect_stats'):
            pod_stats = stats_lib.Stats(pod, full=full)
            click.echo_via_pager('\n\n'.join(pod_stats.to_tables()))
    except pods.Error as err:
        raise click.ClickException(str(err))
    return pod



"""Base installer class."""

from grow.sdk import sdk_utils


class Error(Exception):
    """Base error for installers."""

    def __init__(self, message):
        super(Error, self).__init__(message)
        self.message = message


class MissingPrerequisiteError(Error):
    """Installer is missing a prerequisite."""

    def __init__(self, message, install_commands=None):
        super(MissingPrerequisiteError, self).__init__(message)
        self.install_commands = install_commands


class InstallError(Error):
    """Installer had a error during install."""
    pass


class BaseInstaller(object):
    """Base class for grow installers."""

    KIND = None

    def __init__(self, pod, config):
        self.pod = pod
        self.config = config

    @property
    def pre_install_messages(self):
        """List of messages to display before installing."""
        return ['Installing: {}'.format(self.KIND)]

    @property
    def post_install_messages(self):
        """List of messages to display after installing."""
        return ['Finished: {}'.format(self.KIND)]

    @property
    def should_run(self):
        """Should the installer run?"""
        return True

    def check_prerequisites(self):
        """Check if required prerequisites are installed or available."""
        pass

    def install(self):
        """Run the installer."""
        raise NotImplementedError()

    def subprocess_args(self, shell=False):
        """Arguments for running subprocess commands."""
        return sdk_utils.subprocess_args(self.pod, shell=shell)



"""Dev handler hook."""

from grow.extensions.hooks import base_hook


class DevHandlerHook(base_hook.BaseHook):
    """Hook for dev server handlers."""

    KEY = 'dev_handler'
    NAME = 'Dev Server Handler'

    # pylint: disable=arguments-differ,unused-argument
    def trigger(self, previous_result, routes, *_args, **_kwargs):
        """Trigger the dev handler hook."""
        return previous_result



"""Base class for the jinja environment init hook."""

from grow.extensions.hooks import base_hook


class JinjaEnvInitHook(base_hook.BaseHook):
    """Hook for jinja extension."""

    KEY = 'jinja_env_init'
    NAME = 'Jinja Environment Init'

    # pylint: disable=arguments-differ,unused-argument
    def trigger(self, previous_result, env, *_args, **_kwargs):
        """Trigger the hook."""
        if previous_result:
            return previous_result
        return None



"""Deployment core extension."""

from grow import extensions
from grow.extensions import hooks
from grow.deployments.destinations import git_destination
from grow.deployments.destinations import local
from grow.deployments.destinations import webreview_destination


class DeploymentDeploymentRegisterHook(hooks.DeploymentRegisterHook):
    """Handle the deployment registration hook."""

    # pylint: disable=arguments-differ
    def trigger(self, previous_result, deployments, *_args, **_kwargs):
        """Trigger the deployment destination registration hook."""
        deployments.register_destination(
            webreview_destination.WebReviewDestination)
        deployments.register_destination(
            webreview_destination.LegacyJetwayDestination)
        deployments.register_destination(git_destination.GitDestination)
        deployments.register_destination(local.LocalDestination)


# pylint: disable=abstract-method
class DeploymentExtension(extensions.BaseExtension):
    """Extension for handling core deployment functionality."""

    @property
    def available_hooks(self):
        """Returns the available hook classes."""
        return [DeploymentDeploymentRegisterHook]
